@@ -2,10 +2,12 @@
  *
  * Copyright (C) 2008-2010 Red Hat, Inc.
  *
+ * SPDX-License-Identifier: LGPL-2.1-or-later
+ *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
+ * version 2.1 of the License, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -13,20 +15,18 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General
- * Public License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Public License along with this library; if not, see <http://www.gnu.org/licenses/>.
  *
  * Author: David Zeuthen <davidz@redhat.com>
  */
 
+#ifndef __G_DBUS_CONNECTION_H__
+#define __G_DBUS_CONNECTION_H__
+
 #if !defined (__GIO_GIO_H_INSIDE__) && !defined (GIO_COMPILATION)
 #error "Only <gio/gio.h> can be included directly."
 #endif
 
-#ifndef __G_DBUS_CONNECTION_H__
-#define __G_DBUS_CONNECTION_H__
-
 #include <gio/giotypes.h>
 
 G_BEGIN_DECLS
@@ -35,22 +35,27 @@ G_BEGIN_DECLS
 #define G_DBUS_CONNECTION(o)           (G_TYPE_CHECK_INSTANCE_CAST ((o), G_TYPE_DBUS_CONNECTION, GDBusConnection))
 #define G_IS_DBUS_CONNECTION(o)        (G_TYPE_CHECK_INSTANCE_TYPE ((o), G_TYPE_DBUS_CONNECTION))
 
+GIO_AVAILABLE_IN_ALL
 GType            g_dbus_connection_get_type                   (void) G_GNUC_CONST;
 
 /* ---------------------------------------------------------------------------------------------------- */
 
+GIO_AVAILABLE_IN_ALL
 void              g_bus_get                    (GBusType             bus_type,
                                                 GCancellable        *cancellable,
                                                 GAsyncReadyCallback  callback,
                                                 gpointer             user_data);
+GIO_AVAILABLE_IN_ALL
 GDBusConnection  *g_bus_get_finish             (GAsyncResult        *res,
                                                 GError             **error);
+GIO_AVAILABLE_IN_ALL
 GDBusConnection  *g_bus_get_sync               (GBusType            bus_type,
                                                 GCancellable       *cancellable,
                                                 GError            **error);
 
 /* ---------------------------------------------------------------------------------------------------- */
 
+GIO_AVAILABLE_IN_ALL
 void             g_dbus_connection_new                        (GIOStream              *stream,
                                                                const gchar            *guid,
                                                                GDBusConnectionFlags    flags,
@@ -58,8 +63,10 @@ void             g_dbus_connection_new                        (GIOStream
                                                                GCancellable           *cancellable,
                                                                GAsyncReadyCallback     callback,
                                                                gpointer                user_data);
+GIO_AVAILABLE_IN_ALL
 GDBusConnection *g_dbus_connection_new_finish                 (GAsyncResult           *res,
                                                                GError                **error);
+GIO_AVAILABLE_IN_ALL
 GDBusConnection *g_dbus_connection_new_sync                   (GIOStream              *stream,
                                                                const gchar            *guid,
                                                                GDBusConnectionFlags    flags,
@@ -67,14 +74,17 @@ GDBusConnection *g_dbus_connection_new_sync                   (GIOStream
                                                                GCancellable           *cancellable,
                                                                GError                **error);
 
+GIO_AVAILABLE_IN_ALL
 void             g_dbus_connection_new_for_address            (const gchar            *address,
                                                                GDBusConnectionFlags    flags,
                                                                GDBusAuthObserver      *observer,
                                                                GCancellable           *cancellable,
                                                                GAsyncReadyCallback     callback,
                                                                gpointer                user_data);
+GIO_AVAILABLE_IN_ALL
 GDBusConnection *g_dbus_connection_new_for_address_finish     (GAsyncResult           *res,
                                                                GError                **error);
+GIO_AVAILABLE_IN_ALL
 GDBusConnection *g_dbus_connection_new_for_address_sync       (const gchar            *address,
                                                                GDBusConnectionFlags    flags,
                                                                GDBusAuthObserver      *observer,
@@ -83,54 +93,73 @@ GDBusConnection *g_dbus_connection_new_for_address_sync       (const gchar
 
 /* ---------------------------------------------------------------------------------------------------- */
 
+GIO_AVAILABLE_IN_ALL
 void             g_dbus_connection_start_message_processing   (GDBusConnection    *connection);
+GIO_AVAILABLE_IN_ALL
 gboolean         g_dbus_connection_is_closed                  (GDBusConnection    *connection);
+GIO_AVAILABLE_IN_ALL
 GIOStream       *g_dbus_connection_get_stream                 (GDBusConnection    *connection);
+GIO_AVAILABLE_IN_ALL
 const gchar     *g_dbus_connection_get_guid                   (GDBusConnection    *connection);
+GIO_AVAILABLE_IN_ALL
 const gchar     *g_dbus_connection_get_unique_name            (GDBusConnection    *connection);
+GIO_AVAILABLE_IN_ALL
 GCredentials    *g_dbus_connection_get_peer_credentials       (GDBusConnection    *connection);
 
-GLIB_AVAILABLE_IN_2_34
+GIO_AVAILABLE_IN_2_34
 guint32          g_dbus_connection_get_last_serial            (GDBusConnection    *connection);
 
+GIO_AVAILABLE_IN_ALL
 gboolean         g_dbus_connection_get_exit_on_close          (GDBusConnection    *connection);
+GIO_AVAILABLE_IN_ALL
 void             g_dbus_connection_set_exit_on_close          (GDBusConnection    *connection,
                                                                gboolean            exit_on_close);
+GIO_AVAILABLE_IN_ALL
 GDBusCapabilityFlags  g_dbus_connection_get_capabilities      (GDBusConnection    *connection);
+GIO_AVAILABLE_IN_2_60
+GDBusConnectionFlags  g_dbus_connection_get_flags             (GDBusConnection    *connection);
 
 /* ---------------------------------------------------------------------------------------------------- */
 
+GIO_AVAILABLE_IN_ALL
 void             g_dbus_connection_close                          (GDBusConnection     *connection,
                                                                    GCancellable        *cancellable,
                                                                    GAsyncReadyCallback  callback,
                                                                    gpointer             user_data);
+GIO_AVAILABLE_IN_ALL
 gboolean         g_dbus_connection_close_finish                   (GDBusConnection     *connection,
                                                                    GAsyncResult        *res,
                                                                    GError             **error);
+GIO_AVAILABLE_IN_ALL
 gboolean         g_dbus_connection_close_sync                     (GDBusConnection     *connection,
                                                                    GCancellable        *cancellable,
                                                                    GError             **error);
 
 /* ---------------------------------------------------------------------------------------------------- */
 
+GIO_AVAILABLE_IN_ALL
 void             g_dbus_connection_flush                          (GDBusConnection     *connection,
                                                                    GCancellable        *cancellable,
                                                                    GAsyncReadyCallback  callback,
                                                                    gpointer             user_data);
+GIO_AVAILABLE_IN_ALL
 gboolean         g_dbus_connection_flush_finish                   (GDBusConnection     *connection,
                                                                    GAsyncResult        *res,
                                                                    GError             **error);
+GIO_AVAILABLE_IN_ALL
 gboolean         g_dbus_connection_flush_sync                     (GDBusConnection     *connection,
                                                                    GCancellable        *cancellable,
                                                                    GError             **error);
 
 /* ---------------------------------------------------------------------------------------------------- */
 
+GIO_AVAILABLE_IN_ALL
 gboolean         g_dbus_connection_send_message                   (GDBusConnection     *connection,
                                                                    GDBusMessage        *message,
                                                                    GDBusSendMessageFlags flags,
                                                                    volatile guint32    *out_serial,
                                                                    GError             **error);
+GIO_AVAILABLE_IN_ALL
 void             g_dbus_connection_send_message_with_reply        (GDBusConnection     *connection,
                                                                    GDBusMessage        *message,
                                                                    GDBusSendMessageFlags flags,
@@ -139,9 +168,11 @@ void             g_dbus_connection_send_message_with_reply        (GDBusConnecti
                                                                    GCancellable        *cancellable,
                                                                    GAsyncReadyCallback  callback,
                                                                    gpointer             user_data);
+GIO_AVAILABLE_IN_ALL
 GDBusMessage    *g_dbus_connection_send_message_with_reply_finish (GDBusConnection     *connection,
                                                                    GAsyncResult        *res,
                                                                    GError             **error);
+GIO_AVAILABLE_IN_ALL
 GDBusMessage    *g_dbus_connection_send_message_with_reply_sync   (GDBusConnection     *connection,
                                                                    GDBusMessage        *message,
                                                                    GDBusSendMessageFlags flags,
@@ -152,6 +183,7 @@ GDBusMessage    *g_dbus_connection_send_message_with_reply_sync   (GDBusConnecti
 
 /* ---------------------------------------------------------------------------------------------------- */
 
+GIO_AVAILABLE_IN_ALL
 gboolean  g_dbus_connection_emit_signal                       (GDBusConnection    *connection,
                                                                const gchar        *destination_bus_name,
                                                                const gchar        *object_path,
@@ -159,6 +191,7 @@ gboolean  g_dbus_connection_emit_signal                       (GDBusConnection
                                                                const gchar        *signal_name,
                                                                GVariant           *parameters,
                                                                GError            **error);
+GIO_AVAILABLE_IN_ALL
 void      g_dbus_connection_call                              (GDBusConnection    *connection,
                                                                const gchar        *bus_name,
                                                                const gchar        *object_path,
@@ -171,9 +204,11 @@ void      g_dbus_connection_call                              (GDBusConnection
                                                                GCancellable       *cancellable,
                                                                GAsyncReadyCallback callback,
                                                                gpointer            user_data);
+GIO_AVAILABLE_IN_ALL
 GVariant *g_dbus_connection_call_finish                       (GDBusConnection    *connection,
                                                                GAsyncResult       *res,
                                                                GError            **error);
+GIO_AVAILABLE_IN_ALL
 GVariant *g_dbus_connection_call_sync                         (GDBusConnection    *connection,
                                                                const gchar        *bus_name,
                                                                const gchar        *object_path,
@@ -185,7 +220,10 @@ GVariant *g_dbus_connection_call_sync                         (GDBusConnection
                                                                gint                timeout_msec,
                                                                GCancellable       *cancellable,
                                                                GError            **error);
-GLIB_AVAILABLE_IN_2_30
+
+#ifdef G_OS_UNIX
+
+GIO_AVAILABLE_IN_2_30
 void      g_dbus_connection_call_with_unix_fd_list            (GDBusConnection    *connection,
                                                                const gchar        *bus_name,
                                                                const gchar        *object_path,
@@ -199,12 +237,12 @@ void      g_dbus_connection_call_with_unix_fd_list            (GDBusConnection
                                                                GCancellable       *cancellable,
                                                                GAsyncReadyCallback callback,
                                                                gpointer            user_data);
-GLIB_AVAILABLE_IN_2_30
+GIO_AVAILABLE_IN_2_30
 GVariant *g_dbus_connection_call_with_unix_fd_list_finish     (GDBusConnection    *connection,
                                                                GUnixFDList       **out_fd_list,
                                                                GAsyncResult       *res,
                                                                GError            **error);
-GLIB_AVAILABLE_IN_2_30
+GIO_AVAILABLE_IN_2_30
 GVariant *g_dbus_connection_call_with_unix_fd_list_sync       (GDBusConnection    *connection,
                                                                const gchar        *bus_name,
                                                                const gchar        *object_path,
@@ -219,22 +257,33 @@ GVariant *g_dbus_connection_call_with_unix_fd_list_sync       (GDBusConnection
                                                                GCancellable       *cancellable,
                                                                GError            **error);
 
+#endif /* G_OS_UNIX */
+
 /* ---------------------------------------------------------------------------------------------------- */
 
 
 /**
  * GDBusInterfaceMethodCallFunc:
  * @connection: A #GDBusConnection.
- * @sender: The unique bus name of the remote caller.
+ * @sender: (nullable): The unique bus name of the remote caller, or `NULL` if
+ *     not specified by the caller, e.g. on peer-to-peer connections.
  * @object_path: The object path that the method was invoked on.
- * @interface_name: The D-Bus interface name the method was invoked on.
+ * @interface_name: (nullable): The D-Bus interface name the method was invoked on,
+ *     or `NULL` if not specified by the sender.
  * @method_name: The name of the method that was invoked.
  * @parameters: A #GVariant tuple with parameters.
- * @invocation: A #GDBusMethodInvocation object that can be used to return a value or error.
+ * @invocation: (transfer full): A #GDBusMethodInvocation object that must be used to return a value or error.
  * @user_data: The @user_data #gpointer passed to g_dbus_connection_register_object().
  *
  * The type of the @method_call function in #GDBusInterfaceVTable.
  *
+ * @interface_name may be `NULL` if not specified by the sender, although it’s
+ * encouraged for the sender to set it. If unset, and the object has only one
+ * method (across all interfaces) matching @method_name, that method is invoked.
+ * Otherwise, behaviour is implementation defined. See the
+ * [D-Bus specification](https://dbus.freedesktop.org/doc/dbus-specification.html#message-protocol-types-method).
+ * It is recommended to return [error@Gio.DBusError.UNKNOWN_METHOD].
+ *
  * Since: 2.26
  */
 typedef void (*GDBusInterfaceMethodCallFunc) (GDBusConnection       *connection,
@@ -249,7 +298,8 @@ typedef void (*GDBusInterfaceMethodCallFunc) (GDBusConnection       *connection,
 /**
  * GDBusInterfaceGetPropertyFunc:
  * @connection: A #GDBusConnection.
- * @sender: The unique bus name of the remote caller.
+ * @sender: (nullable): The unique bus name of the remote caller or %NULL if
+ *     not specified by the caller, e.g. on peer-to-peer connections.
  * @object_path: The object path that the method was invoked on.
  * @interface_name: The D-Bus interface name for the property.
  * @property_name: The name of the property to get the value of.
@@ -275,7 +325,8 @@ typedef GVariant *(*GDBusInterfaceGetPropertyFunc) (GDBusConnection       *conne
 /**
  * GDBusInterfaceSetPropertyFunc:
  * @connection: A #GDBusConnection.
- * @sender: The unique bus name of the remote caller.
+ * @sender: (nullable): The unique bus name of the remote caller or %NULL if
+ *     not specified by the caller, e.g. on peer-to-peer connections.
  * @object_path: The object path that the method was invoked on.
  * @interface_name: The D-Bus interface name for the property.
  * @property_name: The name of the property to get the value of.
@@ -307,9 +358,44 @@ typedef gboolean  (*GDBusInterfaceSetPropertyFunc) (GDBusConnection       *conne
  * Virtual table for handling properties and method calls for a D-Bus
  * interface.
  *
- * If you want to handle getting/setting D-Bus properties asynchronously, simply
- * register an object with the <literal>org.freedesktop.DBus.Properties</literal>
- * D-Bus interface using g_dbus_connection_register_object().
+ * Since 2.38, if you want to handle getting/setting D-Bus properties
+ * asynchronously, give %NULL as your get_property() or set_property()
+ * function. The D-Bus call will be directed to your @method_call function,
+ * with the provided @interface_name set to "org.freedesktop.DBus.Properties".
+ *
+ * Ownership of the #GDBusMethodInvocation object passed to the
+ * method_call() function is transferred to your handler; you must
+ * call one of the methods of #GDBusMethodInvocation to return a reply
+ * (possibly empty), or an error. These functions also take ownership
+ * of the passed-in invocation object, so unless the invocation
+ * object has otherwise been referenced, it will be then be freed.
+ * Calling one of these functions may be done within your
+ * method_call() implementation but it also can be done at a later
+ * point to handle the method asynchronously.
+ *
+ * The usual checks on the validity of the calls is performed. For
+ * `Get` calls, an error is automatically returned if the property does
+ * not exist or the permissions do not allow access. The same checks are
+ * performed for `Set` calls, and the provided value is also checked for
+ * being the correct type.
+ *
+ * For both `Get` and `Set` calls, the #GDBusMethodInvocation
+ * passed to the @method_call handler can be queried with
+ * g_dbus_method_invocation_get_property_info() to get a pointer
+ * to the #GDBusPropertyInfo of the property.
+ *
+ * If you have readable properties specified in your interface info,
+ * you must ensure that you either provide a non-%NULL @get_property()
+ * function or provide implementations of both the `Get` and `GetAll`
+ * methods on org.freedesktop.DBus.Properties interface in your @method_call
+ * function. Note that the required return type of the `Get` call is
+ * `(v)`, not the type of the property. `GetAll` expects a return value
+ * of type `a{sv}`.
+ *
+ * If you have writable properties specified in your interface info,
+ * you must ensure that you either provide a non-%NULL @set_property()
+ * function or provide an implementation of the `Set` call. If implementing
+ * the call, you must return the value of type %G_VARIANT_TYPE_UNIT.
  *
  * Since: 2.26
  */
@@ -327,6 +413,7 @@ struct _GDBusInterfaceVTable
   gpointer padding[8];
 };
 
+GIO_AVAILABLE_IN_ALL
 guint            g_dbus_connection_register_object            (GDBusConnection            *connection,
                                                                const gchar                *object_path,
                                                                GDBusInterfaceInfo         *interface_info,
@@ -334,6 +421,23 @@ guint            g_dbus_connection_register_object            (GDBusConnection
                                                                gpointer                    user_data,
                                                                GDestroyNotify              user_data_free_func,
                                                                GError                    **error);
+GIO_DEPRECATED_IN_2_84_FOR(g_dbus_connection_register_object_with_closures2)
+guint            g_dbus_connection_register_object_with_closures (GDBusConnection         *connection,
+                                                                  const gchar             *object_path,
+                                                                  GDBusInterfaceInfo      *interface_info,
+                                                                  GClosure                *method_call_closure,
+                                                                  GClosure                *get_property_closure,
+                                                                  GClosure                *set_property_closure,
+                                                                  GError                 **error);
+GIO_AVAILABLE_IN_2_84
+guint            g_dbus_connection_register_object_with_closures2 (GDBusConnection         *connection,
+                                                                   const gchar             *object_path,
+                                                                   GDBusInterfaceInfo      *interface_info,
+                                                                   GClosure                *method_call_closure,
+                                                                   GClosure                *get_property_closure,
+                                                                   GClosure                *set_property_closure,
+                                                                   GError                 **error);
+GIO_AVAILABLE_IN_ALL
 gboolean         g_dbus_connection_unregister_object          (GDBusConnection            *connection,
                                                                guint                       registration_id);
 
@@ -354,11 +458,11 @@ gboolean         g_dbus_connection_unregister_object          (GDBusConnection
  * specified (ie: to verify that the object path is valid).
  *
  * Hierarchies are not supported; the items that you return should not
- * contain the '/' character.
+ * contain the `/` character.
  *
  * The return value will be freed with g_strfreev().
  *
- * Returns: A newly allocated array of strings for node names that are children of @object_path.
+ * Returns: (array zero-terminated=1) (transfer full): A newly allocated array of strings for node names that are children of @object_path.
  *
  * Since: 2.26
  */
@@ -394,7 +498,7 @@ typedef gchar** (*GDBusSubtreeEnumerateFunc) (GDBusConnection       *connection,
  * remote introspector in the empty array case, but not in the %NULL
  * case.
  *
- * Returns: A %NULL-terminated array of pointers to #GDBusInterfaceInfo, or %NULL.
+ * Returns: (array zero-terminated=1) (nullable) (transfer full): A %NULL-terminated array of pointers to #GDBusInterfaceInfo, or %NULL.
  *
  * Since: 2.26
  */
@@ -411,7 +515,7 @@ typedef GDBusInterfaceInfo ** (*GDBusSubtreeIntrospectFunc) (GDBusConnection
  * @object_path: The object path that was registered with g_dbus_connection_register_subtree().
  * @interface_name: The D-Bus interface name that the method call or property access is for.
  * @node: A node that is a child of @object_path (relative to @object_path) or %NULL for the root of the subtree.
- * @out_user_data: Return location for user data to pass to functions in the returned #GDBusInterfaceVTable (never %NULL).
+ * @out_user_data: (nullable) (not optional): Return location for user data to pass to functions in the returned #GDBusInterfaceVTable.
  * @user_data: The @user_data #gpointer passed to g_dbus_connection_register_subtree().
  *
  * The type of the @dispatch function in #GDBusSubtreeVTable.
@@ -419,7 +523,7 @@ typedef GDBusInterfaceInfo ** (*GDBusSubtreeIntrospectFunc) (GDBusConnection
  * Subtrees are flat.  @node, if non-%NULL, is always exactly one
  * segment of the object path (ie: it never contains a slash).
  *
- * Returns: A #GDBusInterfaceVTable or %NULL if you don't want to handle the methods.
+ * Returns: (nullable): A #GDBusInterfaceVTable or %NULL if you don't want to handle the methods.
  *
  * Since: 2.26
  */
@@ -455,6 +559,7 @@ struct _GDBusSubtreeVTable
   gpointer padding[8];
 };
 
+GIO_AVAILABLE_IN_ALL
 guint            g_dbus_connection_register_subtree           (GDBusConnection            *connection,
                                                                const gchar                *object_path,
                                                                const GDBusSubtreeVTable   *vtable,
@@ -462,6 +567,7 @@ guint            g_dbus_connection_register_subtree           (GDBusConnection
                                                                gpointer                    user_data,
                                                                GDestroyNotify              user_data_free_func,
                                                                GError                    **error);
+GIO_AVAILABLE_IN_ALL
 gboolean         g_dbus_connection_unregister_subtree         (GDBusConnection            *connection,
                                                                guint                       registration_id);
 
@@ -470,7 +576,8 @@ gboolean         g_dbus_connection_unregister_subtree         (GDBusConnection
 /**
  * GDBusSignalCallback:
  * @connection: A #GDBusConnection.
- * @sender_name: The unique bus name of the sender of the signal.
+ * @sender_name: (nullable): The unique bus name of the sender of the signal,
+   or %NULL on a peer-to-peer D-Bus connection.
  * @object_path: The object path that the signal was emitted on.
  * @interface_name: The name of the interface.
  * @signal_name: The name of the signal.
@@ -489,6 +596,7 @@ typedef void (*GDBusSignalCallback) (GDBusConnection  *connection,
                                      GVariant         *parameters,
                                      gpointer          user_data);
 
+GIO_AVAILABLE_IN_ALL
 guint            g_dbus_connection_signal_subscribe           (GDBusConnection     *connection,
                                                                const gchar         *sender,
                                                                const gchar         *interface_name,
@@ -499,9 +607,51 @@ guint            g_dbus_connection_signal_subscribe           (GDBusConnection
                                                                GDBusSignalCallback  callback,
                                                                gpointer             user_data,
                                                                GDestroyNotify       user_data_free_func);
+GIO_AVAILABLE_IN_ALL
 void             g_dbus_connection_signal_unsubscribe         (GDBusConnection     *connection,
                                                                guint                subscription_id);
 
+/**
+ * g_clear_dbus_signal_subscription: (skip)
+ * @subscription_id_pointer: (not optional) (inout): A pointer to either a
+ *    subscription ID obtained from [method@Gio.DBusConnection.signal_subscribe],
+ *    or zero.
+ * @connection: The connection from which the subscription ID was obtained.
+ *    This pointer may be `NULL` or invalid, if the subscription ID is zero.
+ *
+ * If @subscription_id_pointer points to a nonzero subscription ID,
+ * unsubscribe from that D-Bus signal subscription as if via
+ * [method@Gio.DBusConnection.signal_unsubscribe].
+ *
+ * Also set the value pointed to by @subscription_id_pointer to zero,
+ * which signifies it’s no longer a valid subscription ID.
+ *
+ * This convenience function for C code helps to ensure that each signal
+ * subscription is unsubscribed exactly once, similar to
+ * [func@GObject.clear_object] and [func@GObject.clear_signal_handler].
+ *
+ * Since: 2.84
+ */
+GLIB_AVAILABLE_STATIC_INLINE_IN_2_84
+static inline void g_clear_dbus_signal_subscription           (guint               *subscription_id_pointer,
+                                                               GDBusConnection     *connection);
+
+GLIB_AVAILABLE_STATIC_INLINE_IN_2_84
+static inline void
+g_clear_dbus_signal_subscription (guint           *subscription_id_pointer,
+                                  GDBusConnection *connection)
+{
+  guint subscription_id;
+
+  /* Suppress "Not available before" warning */
+  G_GNUC_BEGIN_IGNORE_DEPRECATIONS
+  subscription_id = g_steal_handle_id (subscription_id_pointer);
+  G_GNUC_END_IGNORE_DEPRECATIONS
+
+  if (subscription_id > 0)
+    g_dbus_connection_signal_unsubscribe (connection, subscription_id);
+}
+
 /* ---------------------------------------------------------------------------------------------------- */
 
 /**
@@ -524,7 +674,7 @@ void             g_dbus_connection_signal_unsubscribe         (GDBusConnection
  *                 gboolean         incoming,
  *                 gpointer         user_data)
  * {
- *   /<!-- -->* inspect @message *<!-- -->/
+ *   // inspect @message
  *   return message;
  * }
  * ]|
@@ -557,10 +707,10 @@ void             g_dbus_connection_signal_unsubscribe         (GDBusConnection
  *
  *   error = NULL;
  *   copy = g_dbus_message_copy (message, &error);
- *   /<!-- -->* handle @error being is set *<!-- -->/
+ *   // handle @error being set
  *   g_object_unref (message);
  *
- *   /<!-- -->* modify @copy *<!-- -->/
+ *   // modify @copy
  *
  *   return copy;
  * }
@@ -568,11 +718,11 @@ void             g_dbus_connection_signal_unsubscribe         (GDBusConnection
  * If the returned #GDBusMessage is different from @message and cannot
  * be sent on @connection (it could use features, such as file
  * descriptors, not compatible with @connection), then a warning is
- * logged to <emphasis>standard error</emphasis>. Applications can
+ * logged to standard error. Applications can
  * check this ahead of time using g_dbus_message_to_blob() passing a
  * #GDBusCapabilityFlags value obtained from @connection.
  *
- * Returns: (transfer full) (allow-none): A #GDBusMessage that will be freed with
+ * Returns: (transfer full) (nullable): A #GDBusMessage that will be freed with
  * g_object_unref() or %NULL to drop the message. Passive filter
  * functions can simply return the passed @message object.
  *
@@ -583,11 +733,13 @@ typedef GDBusMessage *(*GDBusMessageFilterFunction) (GDBusConnection *connection
                                                      gboolean         incoming,
                                                      gpointer         user_data);
 
+GIO_AVAILABLE_IN_ALL
 guint g_dbus_connection_add_filter (GDBusConnection            *connection,
                                     GDBusMessageFilterFunction  filter_function,
                                     gpointer                    user_data,
                                     GDestroyNotify              user_data_free_func);
 
+GIO_AVAILABLE_IN_ALL
 void  g_dbus_connection_remove_filter (GDBusConnection    *connection,
                                        guint               filter_id);
 
