@@ -916,3 +916,139 @@
 #define GIO_AVAILABLE_ENUMERATOR_IN_2_76
 #define GIO_AVAILABLE_TYPE_IN_2_76
 #endif
+
+#if GLIB_VERSION_MIN_REQUIRED >= GLIB_VERSION_2_78
+#define GIO_DEPRECATED_IN_2_78 GIO_DEPRECATED
+#define GIO_DEPRECATED_IN_2_78_FOR(f) GIO_DEPRECATED_FOR (f)
+#define GIO_DEPRECATED_MACRO_IN_2_78 GLIB_DEPRECATED_MACRO
+#define GIO_DEPRECATED_MACRO_IN_2_78_FOR(f) GLIB_DEPRECATED_MACRO_FOR (f)
+#define GIO_DEPRECATED_ENUMERATOR_IN_2_78 GLIB_DEPRECATED_ENUMERATOR
+#define GIO_DEPRECATED_ENUMERATOR_IN_2_78_FOR(f) GLIB_DEPRECATED_ENUMERATOR_FOR (f)
+#define GIO_DEPRECATED_TYPE_IN_2_78 GLIB_DEPRECATED_TYPE
+#define GIO_DEPRECATED_TYPE_IN_2_78_FOR(f) GLIB_DEPRECATED_TYPE_FOR (f)
+#else
+#define GIO_DEPRECATED_IN_2_78 _GIO_EXTERN
+#define GIO_DEPRECATED_IN_2_78_FOR(f) _GIO_EXTERN
+#define GIO_DEPRECATED_MACRO_IN_2_78
+#define GIO_DEPRECATED_MACRO_IN_2_78_FOR(f)
+#define GIO_DEPRECATED_ENUMERATOR_IN_2_78
+#define GIO_DEPRECATED_ENUMERATOR_IN_2_78_FOR(f)
+#define GIO_DEPRECATED_TYPE_IN_2_78
+#define GIO_DEPRECATED_TYPE_IN_2_78_FOR(f)
+#endif
+
+#if GLIB_VERSION_MAX_ALLOWED < GLIB_VERSION_2_78
+#define GIO_AVAILABLE_IN_2_78 GIO_UNAVAILABLE (2, 78)
+#define GIO_AVAILABLE_STATIC_INLINE_IN_2_78 GLIB_UNAVAILABLE_STATIC_INLINE (2, 78)
+#define GIO_AVAILABLE_MACRO_IN_2_78 GLIB_UNAVAILABLE_MACRO (2, 78)
+#define GIO_AVAILABLE_ENUMERATOR_IN_2_78 GLIB_UNAVAILABLE_ENUMERATOR (2, 78)
+#define GIO_AVAILABLE_TYPE_IN_2_78 GLIB_UNAVAILABLE_TYPE (2, 78)
+#else
+#define GIO_AVAILABLE_IN_2_78 _GIO_EXTERN
+#define GIO_AVAILABLE_STATIC_INLINE_IN_2_78
+#define GIO_AVAILABLE_MACRO_IN_2_78
+#define GIO_AVAILABLE_ENUMERATOR_IN_2_78
+#define GIO_AVAILABLE_TYPE_IN_2_78
+#endif
+
+#if GLIB_VERSION_MIN_REQUIRED >= GLIB_VERSION_2_80
+#define GIO_DEPRECATED_IN_2_80 GIO_DEPRECATED
+#define GIO_DEPRECATED_IN_2_80_FOR(f) GIO_DEPRECATED_FOR (f)
+#define GIO_DEPRECATED_MACRO_IN_2_80 GLIB_DEPRECATED_MACRO
+#define GIO_DEPRECATED_MACRO_IN_2_80_FOR(f) GLIB_DEPRECATED_MACRO_FOR (f)
+#define GIO_DEPRECATED_ENUMERATOR_IN_2_80 GLIB_DEPRECATED_ENUMERATOR
+#define GIO_DEPRECATED_ENUMERATOR_IN_2_80_FOR(f) GLIB_DEPRECATED_ENUMERATOR_FOR (f)
+#define GIO_DEPRECATED_TYPE_IN_2_80 GLIB_DEPRECATED_TYPE
+#define GIO_DEPRECATED_TYPE_IN_2_80_FOR(f) GLIB_DEPRECATED_TYPE_FOR (f)
+#else
+#define GIO_DEPRECATED_IN_2_80 _GIO_EXTERN
+#define GIO_DEPRECATED_IN_2_80_FOR(f) _GIO_EXTERN
+#define GIO_DEPRECATED_MACRO_IN_2_80
+#define GIO_DEPRECATED_MACRO_IN_2_80_FOR(f)
+#define GIO_DEPRECATED_ENUMERATOR_IN_2_80
+#define GIO_DEPRECATED_ENUMERATOR_IN_2_80_FOR(f)
+#define GIO_DEPRECATED_TYPE_IN_2_80
+#define GIO_DEPRECATED_TYPE_IN_2_80_FOR(f)
+#endif
+
+#if GLIB_VERSION_MAX_ALLOWED < GLIB_VERSION_2_80
+#define GIO_AVAILABLE_IN_2_80 GIO_UNAVAILABLE (2, 80)
+#define GIO_AVAILABLE_STATIC_INLINE_IN_2_80 GLIB_UNAVAILABLE_STATIC_INLINE (2, 80)
+#define GIO_AVAILABLE_MACRO_IN_2_80 GLIB_UNAVAILABLE_MACRO (2, 80)
+#define GIO_AVAILABLE_ENUMERATOR_IN_2_80 GLIB_UNAVAILABLE_ENUMERATOR (2, 80)
+#define GIO_AVAILABLE_TYPE_IN_2_80 GLIB_UNAVAILABLE_TYPE (2, 80)
+#else
+#define GIO_AVAILABLE_IN_2_80 _GIO_EXTERN
+#define GIO_AVAILABLE_STATIC_INLINE_IN_2_80
+#define GIO_AVAILABLE_MACRO_IN_2_80
+#define GIO_AVAILABLE_ENUMERATOR_IN_2_80
+#define GIO_AVAILABLE_TYPE_IN_2_80
+#endif
+
+#if GLIB_VERSION_MIN_REQUIRED >= GLIB_VERSION_2_82
+#define GIO_DEPRECATED_IN_2_82 GIO_DEPRECATED
+#define GIO_DEPRECATED_IN_2_82_FOR(f) GIO_DEPRECATED_FOR (f)
+#define GIO_DEPRECATED_MACRO_IN_2_82 GLIB_DEPRECATED_MACRO
+#define GIO_DEPRECATED_MACRO_IN_2_82_FOR(f) GLIB_DEPRECATED_MACRO_FOR (f)
+#define GIO_DEPRECATED_ENUMERATOR_IN_2_82 GLIB_DEPRECATED_ENUMERATOR
+#define GIO_DEPRECATED_ENUMERATOR_IN_2_82_FOR(f) GLIB_DEPRECATED_ENUMERATOR_FOR (f)
+#define GIO_DEPRECATED_TYPE_IN_2_82 GLIB_DEPRECATED_TYPE
+#define GIO_DEPRECATED_TYPE_IN_2_82_FOR(f) GLIB_DEPRECATED_TYPE_FOR (f)
+#else
+#define GIO_DEPRECATED_IN_2_82 _GIO_EXTERN
+#define GIO_DEPRECATED_IN_2_82_FOR(f) _GIO_EXTERN
+#define GIO_DEPRECATED_MACRO_IN_2_82
+#define GIO_DEPRECATED_MACRO_IN_2_82_FOR(f)
+#define GIO_DEPRECATED_ENUMERATOR_IN_2_82
+#define GIO_DEPRECATED_ENUMERATOR_IN_2_82_FOR(f)
+#define GIO_DEPRECATED_TYPE_IN_2_82
+#define GIO_DEPRECATED_TYPE_IN_2_82_FOR(f)
+#endif
+
+#if GLIB_VERSION_MAX_ALLOWED < GLIB_VERSION_2_82
+#define GIO_AVAILABLE_IN_2_82 GIO_UNAVAILABLE (2, 82)
+#define GIO_AVAILABLE_STATIC_INLINE_IN_2_82 GLIB_UNAVAILABLE_STATIC_INLINE (2, 82)
+#define GIO_AVAILABLE_MACRO_IN_2_82 GLIB_UNAVAILABLE_MACRO (2, 82)
+#define GIO_AVAILABLE_ENUMERATOR_IN_2_82 GLIB_UNAVAILABLE_ENUMERATOR (2, 82)
+#define GIO_AVAILABLE_TYPE_IN_2_82 GLIB_UNAVAILABLE_TYPE (2, 82)
+#else
+#define GIO_AVAILABLE_IN_2_82 _GIO_EXTERN
+#define GIO_AVAILABLE_STATIC_INLINE_IN_2_82
+#define GIO_AVAILABLE_MACRO_IN_2_82
+#define GIO_AVAILABLE_ENUMERATOR_IN_2_82
+#define GIO_AVAILABLE_TYPE_IN_2_82
+#endif
+
+#if GLIB_VERSION_MIN_REQUIRED >= GLIB_VERSION_2_84
+#define GIO_DEPRECATED_IN_2_84 GIO_DEPRECATED
+#define GIO_DEPRECATED_IN_2_84_FOR(f) GIO_DEPRECATED_FOR (f)
+#define GIO_DEPRECATED_MACRO_IN_2_84 GLIB_DEPRECATED_MACRO
+#define GIO_DEPRECATED_MACRO_IN_2_84_FOR(f) GLIB_DEPRECATED_MACRO_FOR (f)
+#define GIO_DEPRECATED_ENUMERATOR_IN_2_84 GLIB_DEPRECATED_ENUMERATOR
+#define GIO_DEPRECATED_ENUMERATOR_IN_2_84_FOR(f) GLIB_DEPRECATED_ENUMERATOR_FOR (f)
+#define GIO_DEPRECATED_TYPE_IN_2_84 GLIB_DEPRECATED_TYPE
+#define GIO_DEPRECATED_TYPE_IN_2_84_FOR(f) GLIB_DEPRECATED_TYPE_FOR (f)
+#else
+#define GIO_DEPRECATED_IN_2_84 _GIO_EXTERN
+#define GIO_DEPRECATED_IN_2_84_FOR(f) _GIO_EXTERN
+#define GIO_DEPRECATED_MACRO_IN_2_84
+#define GIO_DEPRECATED_MACRO_IN_2_84_FOR(f)
+#define GIO_DEPRECATED_ENUMERATOR_IN_2_84
+#define GIO_DEPRECATED_ENUMERATOR_IN_2_84_FOR(f)
+#define GIO_DEPRECATED_TYPE_IN_2_84
+#define GIO_DEPRECATED_TYPE_IN_2_84_FOR(f)
+#endif
+
+#if GLIB_VERSION_MAX_ALLOWED < GLIB_VERSION_2_84
+#define GIO_AVAILABLE_IN_2_84 GIO_UNAVAILABLE (2, 84)
+#define GIO_AVAILABLE_STATIC_INLINE_IN_2_84 GLIB_UNAVAILABLE_STATIC_INLINE (2, 84)
+#define GIO_AVAILABLE_MACRO_IN_2_84 GLIB_UNAVAILABLE_MACRO (2, 84)
+#define GIO_AVAILABLE_ENUMERATOR_IN_2_84 GLIB_UNAVAILABLE_ENUMERATOR (2, 84)
+#define GIO_AVAILABLE_TYPE_IN_2_84 GLIB_UNAVAILABLE_TYPE (2, 84)
+#else
+#define GIO_AVAILABLE_IN_2_84 _GIO_EXTERN
+#define GIO_AVAILABLE_STATIC_INLINE_IN_2_84
+#define GIO_AVAILABLE_MACRO_IN_2_84
+#define GIO_AVAILABLE_ENUMERATOR_IN_2_84
+#define GIO_AVAILABLE_TYPE_IN_2_84
+#endif
